@@ -741,7 +741,7 @@ EXCMD(CMD_jumps,	"jumps",	ex_jumps,
 	EX_TRLBAR|EX_CMDWIN|EX_LOCK_OK,
 	ADDR_NONE),
 EXCMD(CMD_k,		"k",		ex_mark,
-	EX_RANGE|EX_WORD1|EX_TRLBAR|EX_SBOXOK|EX_CMDWIN|EX_LOCK_OK,
+	EX_RANGE|EX_WORD1|EX_TRLBAR|EX_SBOXOK|EX_CMDWIN|EX_LOCK_OK|EX_NONWHITE_OK,
 	ADDR_LINES),
 EXCMD(CMD_keepmarks,	"keepmarks",	ex_wrongmodifier,
 	EX_NEEDARG|EX_EXTRA|EX_NOTRLCOM,
@@ -1679,6 +1679,9 @@ EXCMD(CMD_vimgrep,	"vimgrep",	ex_vimgrep,
 EXCMD(CMD_vimgrepadd,	"vimgrepadd",	ex_vimgrep,
 	EX_RANGE|EX_BANG|EX_NEEDARG|EX_EXTRA|EX_NOTRLCOM|EX_TRLBAR|EX_XFILE|EX_LOCK_OK,
 	ADDR_OTHER),
+EXCMD(CMD_vim9cmd,	"vim9cmd",	ex_wrongmodifier,
+	EX_NEEDARG|EX_EXTRA|EX_NOTRLCOM|EX_CMDWIN|EX_LOCK_OK,
+	ADDR_NONE),
 EXCMD(CMD_vim9script,	"vim9script",	ex_vim9script,
 	EX_WORD1|EX_CMDWIN|EX_LOCK_OK,
 	ADDR_NONE),
@@ -1816,8 +1819,8 @@ EXCMD(CMD_rshift,	">",		ex_operators,
 EXCMD(CMD_at,		"@",		ex_at,
 	EX_RANGE|EX_WHOLEFOLD|EX_EXTRA|EX_TRLBAR|EX_CMDWIN|EX_LOCK_OK|EX_NONWHITE_OK,
 	ADDR_LINES),
-EXCMD(CMD_block,	"{{{{{{{{",	ex_block,  // not found normally
-	0,
+EXCMD(CMD_block,	"{",		ex_block,  // not found normally
+	EX_TRLBAR,
 	ADDR_NONE),
 EXCMD(CMD_endblock,	"}",		ex_endblock,
 	EX_TRLBAR|EX_CMDWIN|EX_LOCK_OK,
